@@ -1,8 +1,8 @@
 /*
  * Compiz configuration system library
  *
- * Copyright (C) 2007  Dennis Kasprzyk <onestone@compiz.org>
- * Copyright (C) 2007  Danny Baumann <dannybaumann@web.de>
+ * Copyright (C) 2007  Dennis Kasprzyk <onestone@opencompositing.org>
+ * Copyright (C) 2007  Danny Baumann <maniac@opencompositing.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -58,6 +58,22 @@ typedef struct _CCSPluginPrivate
     CCSStrExtensionList stringExtensions;
 } CCSPluginPrivate;
 
+typedef struct _CCSSettingsUpgrade
+{
+    char	   *profile;
+    char           *file;
+    char           *domain;
+    unsigned int   num;
+    
+    CCSSettingList changedSettings;
+    CCSSettingList addValueSettings;
+    CCSSettingList clearValueSettings;
+    CCSSettingList replaceFromValueSettings;
+    CCSSettingList replaceToValueSettings;
+} CCSSettingsUpgrade;
+
+Bool ccsCheckForSettingsUpgrade (CCSContext *context);
+
 void ccsLoadPlugins (CCSContext * context);
 void ccsLoadPluginSettings (CCSPlugin * plugin);
 void collateGroups (CCSPluginPrivate * p);
