@@ -1,7 +1,7 @@
 /*
  * Compiz configuration system library
  *
- * Copyright (C) 2007  Dennis Kasprzyk <onestone@compiz.org>
+ * Copyright (C) 2007  Dennis Kasprzyk <onestone@opencompositing.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,8 +23,6 @@
 #include <google/protobuf/io/zero_copy_stream_impl.h>
 #endif
 
-#include <libxml/xpath.h>
-
 extern "C"
 {
 #ifdef HAVE_CONFIG_H
@@ -38,13 +36,17 @@ extern "C"
 #include <sys/stat.h>
 #include <errno.h>
 
+#include <libxslt/transform.h>
+#include <libxslt/xsltutils.h>
+
 #include <locale.h>
 
-#include <compiz-core.h>
 #include <ccs.h>
 #include "ccs-private.h"
 }
 
+#include <core/core.h>
+
 extern int xmlLoadExtDtdDefaultValue;
 
 static const char *
@@ -102,6 +104,7 @@ ccsAddRestrictionToStringInfo (CCSSettingStringInfo *forString,
     restriction = (CCSStrRestriction *) calloc (1, sizeof (CCSStrRestriction));
     if (restriction)
     {
+	restriction->refCount = 1;
 	restriction->name = strdup (name);
 	restriction->value = strdup (value);
 	forString->restriction =
@@ -120,6 +123,7 @@ ccsAddRestrictionToStringExtension (CCSStrExtension *extension,
     restriction = (CCSStrRestriction *) calloc (1, sizeof (CCSStrRestriction));
     if (restriction)
     {
+	restriction->refCount = 1;
 	restriction->name = strdup (name);
 	restriction->value = strdup (value);
 	extension->restriction =
@@ -314,6 +318,7 @@ initListValuePB (CCSSettingValue * v,
 	    if (!val)
 		continue;
 
+	    val->refCount = 1;
 	    val->parent = v->parent;
 	    val->isListChild = TRUE;
 
@@ -394,6 +399,7 @@ initIntInfoPB (CCSSettingInfo * i, const OptionMetadata & option)
 		intDesc = (CCSIntDesc *) calloc (1, sizeof (CCSIntDesc));
 		if (intDesc)
 		{
+		    intDesc->refCount = 1;
 		    intDesc->name = strdup (intDescMetadata.name ().c_str ());
 		    intDesc->value = val;
 		    i->forInt.desc =
@@ -506,15 +512,13 @@ initActionInfoPB (CCSSettingInfo * i, const OptionMetadata & option)
 static void
 addOptionForPluginPB (CCSPlugin * plugin,
 		      const char * name,
-		      Bool isScreen,
-		      unsigned int screen,
 		      const StringList & groups,
 		      const StringList & subgroups,
 		      const OptionMetadata & option)
 {
     CCSSetting *setting;
 
-    if (ccsFindSetting (plugin, name, isScreen, screen))
+    if (ccsFindSetting (plugin, name))
     {
 	fprintf (stderr, "[ERROR]: Option \"%s\" already defined\n", name);
 	return;
@@ -525,10 +529,9 @@ addOptionForPluginPB (CCSPlugin * plugin,
 	return;
 
     setting->parent = plugin;
-    setting->isScreen = isScreen;
-    setting->screenNum = screen;
     setting->isDefault = TRUE;
     setting->name = strdup (name);
+    setting->refCount = 1;
 
     if (!basicMetadata)
     {
@@ -665,7 +668,6 @@ addOptionForPluginPB (CCSPlugin * plugin,
 
 static void
 addOptionFromPB (CCSPlugin * plugin,
-		 Bool isScreen,
 		 const StringList & groups,
 		 const StringList & subgroups,
 		 const OptionMetadata & option)
@@ -680,15 +682,8 @@ addOptionFromPB (CCSPlugin * plugin,
     if (!strlen (name) || readonly)
 	return;
 
-    if (isScreen)
-    {
-	for (unsigned i = 0; i < plugin->context->numScreens; i++)
-	    addOptionForPluginPB (plugin, name, TRUE,
-				  plugin->context->screens[i],
-				  groups, subgroups, option);
-    }
-    else
-	addOptionForPluginPB (plugin, name, FALSE, 0, groups, subgroups, option);
+    addOptionForPluginPB (plugin, name,
+			  groups, subgroups, option);
 }
 
 static void
@@ -697,19 +692,6 @@ initOptionsFromPB (CCSPlugin * plugin,
 {
     int numOpt, i;
 
-    if (pluginPB.has_display ())
-    {
-	const ScreenMetadata &displayPB = pluginPB.display ();
-
-	// Display options
-	numOpt = displayPB.option_size ();
-	for (i = 0; i < numOpt; i++)
-	    addOptionFromPB (plugin, FALSE,
-			     displayPB.group_desc (),
-			     displayPB.subgroup_desc (),
-			     displayPB.option (i));
-    }
-
     if (pluginPB.has_screen ())
     {
 	const ScreenMetadata &screenPB = pluginPB.screen ();
@@ -717,7 +699,7 @@ initOptionsFromPB (CCSPlugin * plugin,
 	// Screen options
 	numOpt = screenPB.option_size ();
 	for (i = 0; i < numOpt; i++)
-	    addOptionFromPB (plugin, TRUE,
+	    addOptionFromPB (plugin,
 			     screenPB.group_desc (),
 			     screenPB.subgroup_desc (),
 			     screenPB.option (i));
@@ -735,7 +717,14 @@ addStringsFromPB (CCSStringList * list,
 	const char *value = (*it).c_str ();
 
 	if (strlen (value))
-	    *list = ccsStringListAppend (*list, strdup (value));
+	{
+	    CCSString *str = (CCSString *) calloc (1, sizeof (CCSString));
+	    
+	    str->value = strdup (value);
+	    str->refCount = 1;
+
+	    *list = ccsStringListAppend (*list, str);
+	}
     }
 }
 
@@ -750,9 +739,7 @@ addStringExtensionFromPB (CCSPlugin * plugin,
     if (!extension)
 	return;
 
-    extension->isScreen = !(extensionPB.has_display () &&
-			    extensionPB.display ());
-
+    extension->refCount = 1;
     extension->restriction = NULL;
 
     extension->basePlugin = strdup (extensionPB.base_plugin ().c_str ());
@@ -839,6 +826,8 @@ addPluginFromPB (CCSContext * context,
     if (!plugin)
 	return;
 
+    plugin->refCount = 1;
+
     pPrivate = (CCSPluginPrivate *) calloc (1, sizeof (CCSPluginPrivate));
     if (!pPrivate)
     {
@@ -903,6 +892,8 @@ addCoreSettingsFromPB (CCSContext * context,
     if (!plugin)
 	return;
 
+    plugin->refCount = 1;
+
     pPrivate = (CCSPluginPrivate *) calloc (1, sizeof (CCSPluginPrivate));
     if (!pPrivate)
     {
@@ -1003,6 +994,7 @@ getOptionType (const char *name)
 	{ "match", TypeMatch },
 	{ "list", TypeList }
     };
+
     for (unsigned i = 0; i < sizeof (map) / sizeof (map[0]); i++)
 	if (strcasecmp (name, map[i].name) == 0)
 	    return map[i].type;
@@ -1511,6 +1503,7 @@ initListValue (CCSSettingValue * v,
 	    if (!val)
 		continue;
 
+	    val->refCount = 1;
 	    val->parent = v->parent;
 	    val->isListChild = TRUE;
 
@@ -1615,6 +1608,7 @@ initIntInfo (CCSSettingInfo * i, xmlNode * node, void * optionPBv)
 			    intDesc = (CCSIntDesc *) calloc (1, sizeof (CCSIntDesc));
 			    if (intDesc)
 			    {
+				intDesc->refCount = 1;
 				intDesc->name = strdup (name);
 				intDesc->value = val;
 				i->forInt.desc =
@@ -1919,7 +1913,7 @@ createProtoBufCacheDir ()
 	metadataCacheDir = cacheBaseDir;
 	if (metadataCacheDir[metadataCacheDir.length () - 1] != '/')
 	    metadataCacheDir += "/";
-	metadataCacheDir += "compizconfig";
+	metadataCacheDir += "compizconfig-1";
 	std::string metadataCacheFileDummy = metadataCacheDir + "/dummy";
 
 	// Create cache dir
@@ -1946,8 +1940,6 @@ addOptionForPlugin (CCSPlugin * plugin,
 		    char * name,
 		    char * type,
 		    Bool isReadonly,
-		    Bool isScreen,
-		    unsigned int screen,
 		    xmlNode * node,
 		    void * groupListPBv,
 		    void * subgroupListPBv,
@@ -1957,7 +1949,7 @@ addOptionForPlugin (CCSPlugin * plugin,
     int num = 0;
     CCSSetting *setting;
 
-    if (ccsFindSetting (plugin, name, isScreen, screen))
+    if (ccsFindSetting (plugin, name))
     {
 	fprintf (stderr, "[ERROR]: Option \"%s\" already defined\n", name);
 	return;
@@ -1971,10 +1963,9 @@ addOptionForPlugin (CCSPlugin * plugin,
 	return;
 
     setting->parent = plugin;
-    setting->isScreen = isScreen;
-    setting->screenNum = screen;
     setting->isDefault = TRUE;
     setting->name = strdup (name);
+    setting->refCount = 1;
 
     if (!basicMetadata)
     {
@@ -2145,7 +2136,6 @@ addOptionForPlugin (CCSPlugin * plugin,
 static void
 addOptionFromXMLNode (CCSPlugin * plugin,
 		      xmlNode * node,
-		      Bool isScreen,
 		      void * groupListPBv,
 		      void * subgroupListPBv,
 		      void * optionPBv)
@@ -2181,18 +2171,9 @@ addOptionFromXMLNode (CCSPlugin * plugin,
 	return;
     }
 
-    if (isScreen)
-    {
-	for (unsigned i = 0; i < plugin->context->numScreens; i++)
-	    addOptionForPlugin (plugin, name, type, isReadonly, TRUE,
-				plugin->context->screens[i], node,
-				groupListPBv, subgroupListPBv, optionPBv);
-    }
-    else
-    {
-	addOptionForPlugin (plugin, name, type, isReadonly, FALSE, 0, node,
-			    groupListPBv, subgroupListPBv, optionPBv);
-    }
+    addOptionForPlugin (plugin, name, type, isReadonly, node,
+			groupListPBv, subgroupListPBv, optionPBv);
+
     free (name);
     free (type);
 
@@ -2201,10 +2182,9 @@ addOptionFromXMLNode (CCSPlugin * plugin,
 }
 
 static void
-initDisplayScreenFromRootNode (CCSPlugin * plugin,
-			       xmlNode * node,
-			       Bool isScreen,
-			       void * pluginPBv)
+initScreenFromRootNode (CCSPlugin * plugin,
+			xmlNode * node,
+			void * pluginPBv)
 {
     xmlNode **nodes;
     xmlNode **optNodes;
@@ -2212,8 +2192,7 @@ initDisplayScreenFromRootNode (CCSPlugin * plugin,
     void *groupListPBv = NULL;
     void *subgroupListPBv = NULL;
 
-    nodes = getNodesFromXPath (node->doc, node,
-			       (isScreen ? "screen" : "display"), &num);
+    nodes = getNodesFromXPath (node->doc, node, "options", &num);
     if (!num)
 	return;
 
@@ -2223,9 +2202,7 @@ initDisplayScreenFromRootNode (CCSPlugin * plugin,
     if (pluginPBv)
     {
 	PluginMetadata *pluginPB = (PluginMetadata *) pluginPBv;
-	screenPB = (isScreen ?
-		    pluginPB->mutable_screen () :
-		    pluginPB->mutable_display ());
+	screenPB = pluginPB->mutable_screen ();
 	groupListPBv = screenPB->mutable_group_desc ();
 	subgroupListPBv = screenPB->mutable_subgroup_desc ();
     }
@@ -2243,7 +2220,7 @@ initDisplayScreenFromRootNode (CCSPlugin * plugin,
 	    if (screenPB)
 		optionPBv = screenPB->add_option ();
     #endif
-	    addOptionFromXMLNode (plugin, optNodes[i], isScreen,
+	    addOptionFromXMLNode (plugin, optNodes[i],
 				  groupListPBv, subgroupListPBv, optionPBv);
 	}
 
@@ -2257,11 +2234,8 @@ initOptionsFromRootNode (CCSPlugin * plugin,
 			 xmlNode * node,
 			 void * pluginPBv)
 {
-    // For display
-    initDisplayScreenFromRootNode (plugin, node, FALSE, pluginPBv);
-
-    // For screen
-    initDisplayScreenFromRootNode (plugin, node, TRUE, pluginPBv);
+    // For all optiond
+    initScreenFromRootNode (plugin, node, pluginPBv);
 }
 
 static void
@@ -2282,7 +2256,12 @@ addStringsFromPath (CCSStringList * list,
 
 	    if (value && strlen (value))
 	    {
-		*list = ccsStringListAppend (*list, value);
+		CCSString *str = (CCSString *) calloc (1, sizeof (CCSString));
+		
+		str->value = value;
+		str->refCount = 1;
+
+		*list = ccsStringListAppend (*list, str);
 #ifdef USE_PROTOBUF
 		if (stringListPBv)
 		    ((StringList *) stringListPBv)->Add ()->assign (value);
@@ -2306,20 +2285,13 @@ addStringExtensionFromXMLNode (CCSPlugin * plugin,
     CCSStrExtension *extension;
     char *name;
     char *value;
-    char *isDisplay;
     void * stringListPBv = NULL;
 
     extension = (CCSStrExtension *) calloc (1, sizeof (CCSStrExtension));
     if (!extension)
 	return;
 
-    isDisplay = getStringFromXPath (node->doc, node, "@display");
-
-    extension->isScreen = !(isDisplay && !strcmp (isDisplay, "true"));
-
-    if (isDisplay)
-	free (isDisplay);
-
+    extension->refCount = 1;
     extension->restriction = NULL;
 
     extension->basePlugin = getStringFromXPath (node->doc, node, "@base_plugin");
@@ -2331,7 +2303,6 @@ addStringExtensionFromXMLNode (CCSPlugin * plugin,
     if (extensionPBv)
     {
 	extensionPB = (ExtensionMetadata *) extensionPBv;
-	extensionPB->set_display (!extension->isScreen);
 	extensionPB->set_base_plugin (extension->basePlugin);
 	stringListPBv = extensionPB->mutable_base_option ();
     }
@@ -2502,6 +2473,8 @@ addPluginFromXMLNode (CCSContext * context,
     if (!plugin)
 	return FALSE;
 
+    plugin->refCount = 1;
+
     pPrivate = (CCSPluginPrivate *) calloc (1, sizeof (CCSPluginPrivate));
     if (!pPrivate)
     {
@@ -2565,6 +2538,8 @@ addCoreSettingsFromXMLNode (CCSContext * context,
     if (!plugin)
 	return FALSE;
 
+    plugin->refCount = 1;
+
     pPrivate = (CCSPluginPrivate *) calloc (1, sizeof (CCSPluginPrivate));
     if (!pPrivate)
     {
@@ -2577,7 +2552,7 @@ addCoreSettingsFromXMLNode (CCSContext * context,
     if (file)
 	pPrivate->xmlFile = strdup (file);
 
-    pPrivate->xmlPath = strdup ("/compiz/core");
+    pPrivate->xmlPath = strdup ("/compiz/plugin[@name='core']");
     plugin->context = context;
     plugin->name = strdup ("core");
     plugin->category = strdup ("General");
@@ -2710,7 +2685,7 @@ loadPluginFromXML (CCSContext * context,
     int num;
     Bool success = FALSE;
 
-    nodes = getNodesFromXPath (doc, NULL, "/compiz/core", &num);
+    nodes = getNodesFromXPath (doc, NULL, "/compiz/plugin[@name='core']", &num);
     if (num)
     {
 	success = addCoreSettingsFromXMLNode (context, nodes[0], filename,
@@ -2749,7 +2724,6 @@ static void
 loadPluginFromXMLFile (CCSContext * context, char *xmlName, char *xmlDirPath)
 {
     char *xmlFilePath = NULL;
-    char *pbFilePath = NULL;
     void *pluginInfoPBv = NULL;
 
     asprintf (&xmlFilePath, "%s/%s", xmlDirPath, xmlName);
@@ -2761,6 +2735,7 @@ loadPluginFromXMLFile (CCSContext * context, char *xmlName, char *xmlDirPath)
 
 #ifdef USE_PROTOBUF
     char *name = NULL;
+    char *pbFilePath = NULL;
     struct stat xmlStat;
     Bool removePB = FALSE;
 
@@ -2811,9 +2786,9 @@ loadPluginFromXMLFile (CCSContext * context, char *xmlName, char *xmlDirPath)
 		else
 		    addPluginFromPB (context, persistentPluginBriefPB.info (),
 				     pbFilePath, xmlFilePath);
-		
+    
 		updatePBFilePath (context, name, pbFilePath);
-		
+    
 		free (xmlFilePath);
 		free (pbFilePath);
 		free (name);
@@ -2824,6 +2799,7 @@ loadPluginFromXMLFile (CCSContext * context, char *xmlName, char *xmlDirPath)
 		removePB = TRUE;
 	    }
 	}
+
 	persistentPluginBriefPB.Clear ();
 	pluginInfoPBv = persistentPluginBriefPB.mutable_info ();
     }
@@ -2907,6 +2883,8 @@ addPluginNamed (CCSContext * context, char *name)
     if (!plugin)
 	return;
 
+    plugin->refCount = 1;
+
     pPrivate = (CCSPluginPrivate *) calloc (1, sizeof (CCSPluginPrivate));
     if (!pPrivate)
     {
@@ -2991,8 +2969,7 @@ ccsLoadPlugin (CCSContext * context, char *name)
 	char *home = getenv ("HOME");
 	if (home && strlen (home))
 	{
-	    char *home = getenv ("HOME");
-	    asprintf (&xmlDirPath, "%s/.compiz/metadata", home);
+	    asprintf (&xmlDirPath, "%s/.compiz-1/metadata", home);
 	    if (xmlDirPath)
 	    {
 		loadPluginFromXMLFile (context, xmlName, xmlDirPath);
@@ -3020,7 +2997,7 @@ ccsLoadPlugins (CCSContext * context)
     if (home && strlen (home))
     {
 	char *homeplugins = NULL;
-	asprintf (&homeplugins, "%s/.compiz/metadata", home);
+	asprintf (&homeplugins, "%s/.compiz-1/metadata", home);
 	if (homeplugins)
 	{
 	    loadPluginsFromXMLFiles (context, homeplugins);
@@ -3032,7 +3009,7 @@ ccsLoadPlugins (CCSContext * context)
     if (home && strlen (home))
     {
 	char *homeplugins = NULL;
-	asprintf (&homeplugins, "%s/.compiz/plugins", home);
+	asprintf (&homeplugins, "%s/.compiz-1/plugins", home);
 	if (homeplugins)
 	{
 	    loadPluginsFromName (context, homeplugins);
@@ -3079,10 +3056,10 @@ void
 ccsLoadPluginSettings (CCSPlugin * plugin)
 {
     Bool ignoreXML = FALSE;
-    Bool loadedAtLeastBriefPB = FALSE;
     void *pluginPBToWrite = NULL;
 
 #ifdef USE_PROTOBUF
+    Bool loadedAtLeastBriefPB = FALSE;
     initPBLoading ();
 #endif
 
