@@ -1,8 +1,8 @@
 /*
  * Compiz configuration system library
  *
- * Copyright (C) 2007  Dennis Kasprzyk <onestone@compiz.org>
- * Copyright (C) 2007  Danny Baumann <dannybaumann@web.de>
+ * Copyright (C) 2007  Dennis Kasprzyk <onestone@opencompositing.org>
+ * Copyright (C) 2007  Danny Baumann <maniac@opencompositing.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,7 +27,7 @@
 
 #include "ccs-private.h"
 
-#define SETTINGPATH "compiz/compizconfig"
+#define SETTINGPATH "compiz-1/compizconfig"
 
 static char*
 getConfigFileName (void)
@@ -65,9 +65,9 @@ getSectionName (void)
 	return section;
     }
 
-    profile = getenv ("MATE_DESKTOP_SESSION_ID");
+    profile = getenv ("GNOME_DESKTOP_SESSION_ID");
     if (profile && strlen (profile))
-	return strdup ("mate_session");
+	return strdup ("gnome_session");
 
     profile = getenv ("KDE_SESSION_VERSION");
     if (profile && strlen (profile) && strcasecmp (profile, "4") == 0)
